<?php
declare(strict_types=1);

use Migrations\AbstractMigration;

/**
 * Class HostAndServicegroups
 *
 * Add tables to save host and service group information
 *
 * Created:
 * oitc bake migration HostAndServicegroups
 *
 * Run migration:
 * oitc migrations migrate
 */
class HostAndServicegroups extends AbstractMigration {

    /**
     * Whether the tables created in this migration
     * should auto-create an `id` field or not
     *
     * This option is global for all tables created in the migration file.
     * If you set it to false, you have to manually add the primary keys for your
     * tables using the Migrations\Table::addPrimaryKey() method
     *
     * @var bool
     */
    public $autoId = false;

    /**
     * Change Method.
     *
     * More information on this method is available here:
     * https://book.cakephp.org/phinx/0/en/migrations.html#the-change-method
     * @return void
     */
    public function change(): void {

        $this->table('hostgroups')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 11,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('uuid', 'string', [
                'default' => null,
                'limit'   => 37,
                'null'    => false,
            ])
            ->addColumn('description', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => false,
            ])
            ->addIndex(
                [
                    'uuid',
                ],
                ['unique' => true]
            )
            ->create();

        $this->table('hostgroup_members')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 11,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('hostgroup_id', 'integer', [
                'default' => null,
                'limit'   => 11,
                'null'    => false,
            ])
            ->addColumn('host_uuid', 'string', [
                'default' => null,
                'limit'   => 37,
                'null'    => false
            ])
            ->addIndex(
                [
                    'hostgroup_id',
                ]
            )
            ->addIndex(
                [
                    'host_uuid',
                ]
            )
            ->create();

        $this->table('servicegroups')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 11,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('uuid', 'string', [
                'default' => null,
                'limit'   => 37,
                'null'    => false,
            ])
            ->addColumn('description', 'string', [
                'default' => null,
                'limit'   => 255,
                'null'    => false,
            ])
            ->addIndex(
                [
                    'uuid',
                ],
                ['unique' => true]
            )
            ->create();

        $this->table('servicegroup_members')
            ->addColumn('id', 'integer', [
                'autoIncrement' => true,
                'default'       => null,
                'limit'         => 11,
                'null'          => false,
            ])
            ->addPrimaryKey(['id'])
            ->addColumn('servicegroup_id', 'integer', [
                'default' => null,
                'limit'   => 11,
                'null'    => false,
            ])
            ->addColumn('service_uuid', 'string', [
                'default' => null,
                'limit'   => 37,
                'null'    => false
            ])
            ->addIndex(
                [
                    'servicegroup_id',
                ]
            )
            ->addIndex(
                [
                    'service_uuid',
                ]
            )
            ->create();

    }
}
