<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace MapModule\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * MapsToRotations Model
 *
 * @property \MapModule\Model\Table\MapsTable&\Cake\ORM\Association\BelongsTo $Maps
 * @property \MapModule\Model\Table\RotationsTable&\Cake\ORM\Association\BelongsTo $Rotations
 *
 * @method \MapModule\Model\Entity\MapsToRotation newEmptyEntity()
 * @method \MapModule\Model\Entity\MapsToRotation newEntity(array $data, array $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation[] newEntities(array $data, array $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation get($primaryKey, $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \MapModule\Model\Entity\MapsToRotation[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 *
 * This Table is used for the TRUNCATE TABLE in the MappingImportCommand.php
 */
class MapsToRotationsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('maps_to_rotations');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->belongsTo('Maps', [
            'foreignKey' => 'map_id',
            'joinType'   => 'INNER',
            'className'  => 'MapModule.Maps',
        ]);
        $this->belongsTo('Rotations', [
            'foreignKey' => 'rotation_id',
            'joinType'   => 'INNER',
            'className'  => 'MapModule.Rotations',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->existsIn(['map_id'], 'Maps'), ['errorField' => 'map_id']);
        $rules->add($rules->existsIn(['rotation_id'], 'Rotations'), ['errorField' => 'rotation_id']);

        return $rules;
    }
}
