<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

declare(strict_types=1);

namespace MapModule\Model\Table;

use App\Lib\Traits\PaginationAndScrollIndexTrait;
use Cake\Datasource\EntityInterface;
use Cake\ORM\Association\HasMany;
use Cake\ORM\Behavior\TimestampBehavior;
use Cake\ORM\Query;
use Cake\ORM\Table;
use Cake\Validation\Validator;
use itnovum\openITCOCKPIT\Database\PaginateOMat;
use MapModule\Model\Entity\Rotation;

/**
 * Rotations Model
 *
 * @property RotationsTable&HasMany $MapsToRotations
 *
 * @method Rotation get($primaryKey, $options = [])
 * @method Rotation newEntity($data = null, array $options = [])
 * @method Rotation[] newEntities(array $data, array $options = [])
 * @method Rotation|false save(EntityInterface $entity, $options = [])
 * @method Rotation saveOrFail(EntityInterface $entity, $options = [])
 * @method Rotation patchEntity(EntityInterface $entity, array $data, array $options = [])
 * @method Rotation[] patchEntities($entities, array $data, array $options = [])
 * @method Rotation findOrCreate($search, callable $callback = null, $options = [])
 *
 * @mixin TimestampBehavior
 */
class RotationsTable extends Table {

    use PaginationAndScrollIndexTrait;

    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('rotations');
        $this->setDisplayField('name');
        $this->setPrimaryKey('id');

        $this->addBehavior('Timestamp');

        $this->belongsToMany('Maps', [
            'className'        => 'MapModule.Maps',
            'foreignKey'       => 'rotation_id',
            'targetForeignKey' => 'map_id',
            'joinTable'        => 'maps_to_rotations',
            'saveStrategy'     => 'replace',
        ]);
    }

    /**
     * Default validation rules.
     *
     * @param Validator $validator Validator instance.
     * @return Validator
     */
    public function validationDefault(Validator $validator): Validator {
        return $validator;
    }

    /**
     * @param $id
     * @return bool
     */
    public function existsById($id) {
        return $this->exists(['Rotations.id' => $id]);
    }

    /**
     * @param array $indexFilter
     * @param array $orderForPaginator
     * @param int|null $limit
     * @param PaginateOMat|null $PaginateOMat
     * @return array
     */
    public function getAll(array $indexFilter, array $orderForPaginator, int $limit = null, PaginateOMat $PaginateOMat = null) {

        $query = $this->find()
            ->contain([
                'Maps'
            ]);
        if (!empty($indexFilter)) {
            $query->where($indexFilter);
        }

        if ($limit !== null) {
            $query->limit($limit);
        }

        $queryResult = $query->order($orderForPaginator)
            ->group(['Rotations.id'])
            ->enableAutoFields(true)
            ->all();

        if (empty($queryResult)) {
            $result = [];
        } else {
            if ($PaginateOMat === null) {
                $result = $query->toArray();
            } else {
                if ($PaginateOMat->useScroll()) {
                    $result = $this->scrollCake4($query, $PaginateOMat->getHandler());
                } else {
                    $result = $this->paginateCake4($query, $PaginateOMat->getHandler());
                }
            }
        }

        return $result;
    }
}
