<?php
/**
 * CakePHP(tm) : Rapid Development Framework (http://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @link      http://cakephp.org CakePHP(tm) Project
 * @since     0.2.9
 * @license   http://www.opensource.org/licenses/mit-license.php MIT License
 */
declare(strict_types=1);

namespace App\Controller;

use App\Lib\SystemConfig;

/**
 * Static content controller
 *
 * This controller will render views from Template/Pages/
 *
 * @link http://book.cakephp.org/3.0/en/controllers/pages-controller.html
 */
class PagesController extends AppController {

    public function index() {

        if ($this->isApiRequest() === false) {
            $user = $this->getUser();
            $showstatsinmenu = (bool)$user->get('showstatsinmenu');

            $userFullName = sprintf('%s %s', $user->get('firstname'), $user->get('lastname'));
            if ($userFullName === ' ') {
                $userFullName = $user->get('username');
            }


            $userImage = '/img/fallback_user.png';
            if ($user->get('image') != null && $user->get('image') != '') {
                if (file_exists(WWW_ROOT . 'img' . DS . 'userimages' . DS . $user->get('image'))) {
                    $userImage = '/img/userimages' . DS . $user->get('image');
                }
            }

            $SystemConfig = new SystemConfig();
            $config = [
                'title'      => $SystemConfig->getTitle(),
                'systemname' => $SystemConfig->getSystemname()
            ];

            $this->set('config', $config);
            $this->set('userFullName', $userFullName);
            $this->set('userImage', $userImage);
            $this->set('showstatsinmenu', $showstatsinmenu);
            // Ship the HTML layout to load JS and CSS files
            $this->viewBuilder()->setLayout('app_frame');
        }
    }

    public function configuration() {
        if (!$this->isApiRequest()) {
            $this->autoRender = false;
            return;
        }

        $SystemConfig = new SystemConfig();
        $config = [];
        $config['auth'] = $this->Auth->user();

        $this->set(compact('config'));
        $this->viewBuilder()->setOption('serialize', ['config']);
    }
}
