<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use App\Model\Table\ApikeysTable;
use App\Model\Table\UsersTable;
use Authorization\Identity;
use Cake\Cache\Cache;
use Cake\Http\Exception\BadRequestException;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Core\System\FileUploadSize;
use itnovum\openITCOCKPIT\Core\Views\Apikey;

/**
 * Class ProfileController
 * @package App\Controller
 */
class ProfileController extends AppController {

    public function edit() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var Identity $User */
        $User = $this->getUser();

        /** @var UsersTable $UsersTable */
        $UsersTable = TableRegistry::getTableLocator()->get('Users');

        if (!$UsersTable->existsById($User->get('id'))) {
            throw new NotFoundException(__('User not found'));
        }

        $user = $UsersTable->get($User->get('id'));

        $FileUploadSize = new FileUploadSize();

        if ($this->request->is('get') && $this->isAngularJsRequest()) {
            //Return profile information
            $this->set('user', $user);
            $this->set('maxUploadLimit', $FileUploadSize->toArray());
            $this->viewBuilder()->setOption('serialize', ['user', 'maxUploadLimit']);
            return;
        }

        if ($this->request->is('post') || $this->request->is('put')) {
            $data = $this->request->getData('User', []);

            $user->setAccess('id', false);
            $user->setAccess('username', false);

            //prevent multiple hash of password
            if ($data['password'] === '' && $data['confirm_password'] === '') {
                unset($data['password']);
                unset($data['confirm_password']);
            }

            $user = $UsersTable->patchEntity($user, $data);
            $UsersTable->save($user);
            if ($user->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $user->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            //Update user information in $_SESSION
            Cache::clearAll();
            $session = $this->request->getSession();
            $session->write('Auth', $UsersTable->get($User->get('id')));

            $this->set('user', $user);
            $this->viewBuilder()->setOption('serialize', ['user']);
        }
    }

    /****************************
     *       AJAX METHODS       *
     ****************************/

    public function changePassword() {
        /** @var Identity $User */
        $User = $this->getUser();

        /** @var UsersTable $UsersTable */
        $UsersTable = TableRegistry::getTableLocator()->get('Users');
        $Hasher = $UsersTable->getPasswordHasher();

        $user = $UsersTable->get($User->get('id'));

        if ($user === null) {
            throw new NotFoundException();
        }

        $data = $this->request->getData('Password');

        if (!$Hasher->check($data['current_password'], $user->get('password'))) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', [
                'current_password' => [
                    __('Current password is incorrect')
                ]
            ]);
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        $user = $UsersTable->patchEntity($user, $data);
        $UsersTable->save($user);
        if ($user->hasErrors()) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', $user->getErrors());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        $session = $this->request->getSession();
        $session->write('Auth', $UsersTable->get($User->get('id')));

        $this->set('message', __('Password changed successfully.'));
        $this->viewBuilder()->setOption('serialize', ['message']);
    }

    public function upload_profile_icon() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var Identity $User */
        $User = $this->getUser();

        /** @var $UsersTable UsersTable */
        $UsersTable = TableRegistry::getTableLocator()->get('Users');
        $user = $UsersTable->get($User->get('id'));

        if ($user === null) {
            throw new NotFoundException();
        }

        /***** Change users profile image *****/
        if (!is_dir(WWW_ROOT . 'img' . DS . 'userimages')) {
            mkdir(WWW_ROOT . 'img' . DS . 'userimages');
        }

        if (isset($_FILES['Picture']['tmp_name']) && isset($_FILES['Picture']['name'])) {
            $filename = $UsersTable->uploadProfilePicture();
            if ($filename) {
                //Delete old image

                $oldImage = $user->get('image');
                if ($oldImage !== '' && $oldImage !== null) {
                    $oldImageFull = WWW_ROOT . 'img' . DS . 'userimages' . DS . $oldImage;
                    if (file_exists($oldImageFull) && !is_dir($oldImageFull)) {
                        unlink($oldImageFull);
                    }
                }

                $user->set('image', $filename);
                //prevent multiple hash of password
                unset($user->password);
                $UsersTable->save($user);
                if ($user->hasErrors()) {
                    $this->response = $this->response->withStatus(400);
                    $this->set('error', $user->getErrors());
                    $this->viewBuilder()->setOption('serialize', ['error']);
                    return;
                }

                //Update cached data in user identity / current session
                $session = $this->request->getSession();
                $UserEntity = $session->read('Auth');
                if ($UserEntity instanceof \App\Model\Entity\User) {
                    $UserEntity->set('image', $filename);
                }

                $this->response = $this->response->withStatus(200);
                $this->set('success', true);
                $this->set('message', __('File Upload success!'));
                $this->viewBuilder()->setOption('serialize', ['success', 'message']);
                return;
            }

            $this->response = $this->response->withStatus(400);
            $this->set('error', __('Could not save image data, may be wrong data type. Allowed types are .png, .jpg and .gif'));
            $this->set('message', __('Could not save image data, may be wrong data type. Allowed types are .png, .jpg and .gif'));
            $this->viewBuilder()->setOption('serialize', ['error', 'message']);
            return;
        }
        $this->set('user', []);
        $this->viewBuilder()->setOption('serialize', ['user']);

    }

    public function apikey() {
        if (!$this->isApiRequest()) {
            throw new MethodNotAllowedException('Only API requests.');
        }

        /** @var Identity $User */
        $User = $this->getUser();

        /** @var ApikeysTable $ApikeysTable */
        $ApikeysTable = TableRegistry::getTableLocator()->get('Apikeys');

        if ($this->request->is('get')) {
            $id = $this->request->getQuery('id');
            if (is_numeric($id)) {
                //Get an api key by id and user_id
                if (!$ApikeysTable->existsById($id)) {
                    throw new NotFoundException(__('Invalid API key'));
                }
                $apikey = $ApikeysTable->getApikeyByIdAndUserId(
                    $id,
                    $User->get('id')
                );
                $Apikey = new Apikey($apikey);
                $this->set('apikey', $Apikey->toArray());
                $this->viewBuilder()->setOption('serialize', ['apikey']);
                return;
            }

            //Return all api keys of the user
            $apikeysResult = $ApikeysTable->getAllapiKeysByUserId($User->get('id'));

            $apikeys = [];
            foreach ($apikeysResult as $apikey) {
                $Apikey = new Apikey($apikey);
                $apikeys[] = $Apikey->toArray();
            }

            $this->set('apikeys', $apikeys);
            $this->viewBuilder()->setOption('serialize', ['apikeys']);
            return;
        }

        if ($this->request->is('post')) {
            //Update an api key by id
            $data = $this->request->getData('Apikey', []);
            if (isset($data['id'])) {
                $id = $data['id'];

                if (!$ApikeysTable->existsById($id)) {
                    throw new NotFoundException(__('Invalid API key'));
                }
                $apikey = $ApikeysTable->get($id);
                $apikey = $ApikeysTable->patchEntity($apikey, $data);

                $ApikeysTable->save($apikey);
                if ($apikey->hasErrors()) {
                    $this->response = $this->response->withStatus(400);
                    $this->set('error', $apikey->getErrors());
                    $this->viewBuilder()->setOption('serialize', ['error']);
                    return;
                } else {
                    $this->set('message', __('API key updated successfully'));
                    $this->viewBuilder()->setOption('serialize', ['message']);
                    return;
                }
            }

        }
    }

    public function create_apikey() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        /** @var ApikeysTable $ApikeysTable */
        $ApikeysTable = TableRegistry::getTableLocator()->get('Apikeys');

        if ($this->request->is('get')) {
            //Generate new API key
            $apikey = $ApikeysTable->generateApiKey();

            $newApiKey = [
                'key'  => $apikey,
                'time' => time()
            ];

            $session = $this->request->getSession();
            $session->write('latest_api_key', $newApiKey);

            $this->set('apikey', $apikey);
            $this->viewBuilder()->setOption('serialize', ['apikey']);
            return;
        }

        if ($this->request->is('post')) {
            $User = $this->getUser();
            //Save new API key
            //Resolve ITC-2170
            $session = $this->request->getSession();

            $newApiKey = $session->read('latest_api_key');
            $session->delete('latest_api_key');

            if (!isset($newApiKey['key']) || !$newApiKey['time']) {
                throw new BadRequestException();
            }
            //Is API-Key older than 5 minutes?
            if ($newApiKey['time'] < (time() - 5 * 60)) {
                throw new BadRequestException('API key expired');
            }

            $apikey = [
                'apikey'      => $newApiKey['key'],
                'description' => $this->request->getData('Apikey.description', ''),
                'user_id'     => $User->get('id')
            ];

            $apikeyEntity = $ApikeysTable->newEmptyEntity();
            $apikeyEntity = $ApikeysTable->patchEntity($apikeyEntity, $apikey);

            $ApikeysTable->save($apikeyEntity);
            if ($apikeyEntity->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $apikeyEntity->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
            } else {
                $this->set('message', __('API key created successfully'));
                $this->viewBuilder()->setOption('serialize', ['message']);
            }
            return;
        }
    }

    public function delete_apikey($id = null) {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }


        $User = $this->getUser();

        /** @var ApikeysTable $ApikeysTable */
        $ApikeysTable = TableRegistry::getTableLocator()->get('Apikeys');

        $apikey = $ApikeysTable->getApikeyByIdAndUserId($id, $User->get('id'));

        if (empty($apikey)) {
            throw new NotFoundException('Invalide API key');
        }

        if ($ApikeysTable->delete($apikey)) {
            $this->set('message', __('Api key deleted successfully'));
            $this->viewBuilder()->setOption('serialize', ['message']);
            return;
        }

        $this->response = $this->response->withStatus(400);
        $this->set('message', __('Could not delete api key'));
        $this->viewBuilder()->setOption('serialize', ['message']);

    }

    public function deleteImage() {
        if (!$this->isApiRequest()) {
            //Only ship HTML template for angular
            return;
        }

        $User = $this->getUser();

        /** @var $UsersTable UsersTable */
        $UsersTable = TableRegistry::getTableLocator()->get('Users');
        $user = $UsersTable->get($User->get('id'));

        //prevent multiple hash of password
        unset($user->password);

        if ($user->image != null && $user->image != '') {
            if (file_exists(WWW_ROOT . 'img' . DS . 'userimages' . DS . $user->image)) {
                unlink(WWW_ROOT . 'img' . DS . 'userimages' . DS . $user->image);
            }
        }

        $user->image = null;

        $UsersTable->save($user);
        if ($user->hasErrors()) {
            $this->response = $this->response->withStatus(400);
            $this->set('error', $user->getErrors());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        //Update cached data in user identity / current session
        $session = $this->request->getSession();
        $UserEntity = $session->read('Auth');
        if ($UserEntity instanceof \App\Model\Entity\User) {
            $UserEntity->set('image', null);
        }

        $this->response = $this->response->withStatus(200);
        $this->set('success', true);
        $this->set('message', __('File deleted sucessfully'));
        $this->viewBuilder()->setOption('serialize', ['success', 'message']);
        return;
    }

    public function edit_apikey() {
        //Only ship HTML template
        return;
    }

}
