<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Controller;

use Acl\Model\Table\AcosTable;
use Acl\Model\Table\ArosTable;
use App\Filter\GenericFilter;
use App\Lib\AclDependencies;
use App\Lib\SystemConfig;
use App\Model\Table\ArosAcosTable;
use App\Model\Table\UsergroupsTable;
use Cake\Cache\Cache;
use Cake\Http\Exception\MethodNotAllowedException;
use Cake\Http\Exception\NotFoundException;
use Cake\ORM\Table;
use Cake\ORM\TableRegistry;
use itnovum\openITCOCKPIT\Database\PaginateOMat;

/**
 * Class UsergroupsController
 * @package App\Controller
 */
class UsergroupsController extends AppController {

    public function index() {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }

        $GenericFilter = new GenericFilter($this->request);
        $GenericFilter->setFilters([
            'like' => [
                'Usergroups.name',
                'Usergroups.description'
            ]
        ]);

        $PaginateOMat = new PaginateOMat($this, $this->isScrollRequest(), $GenericFilter->getPage());

        /** @var UsergroupsTable $UsergroupsTable */
        $UsergroupsTable = TableRegistry::getTableLocator()->get('Usergroups');
        $allUsergroups = $UsergroupsTable->getUsergroups($PaginateOMat, $GenericFilter);

        $this->set('allUsergroups', $allUsergroups);
        $this->viewBuilder()->setOption('serialize', ['allUsergroups']);
    }

    /**
     * @throws \Exception
     */
    public function add() {
        if (!$this->isAngularJsRequest()) {
            //Only ship HTML Template
            return;
        }

        /** @var AcosTable $AcosTable */
        $AcosTable = TableRegistry::getTableLocator()->get('Acl.Acos');
        if ($this->request->is('get') && $this->isJsonRequest()) {
            $acos = $AcosTable->find('threaded')
                ->disableHydration()
                ->all();
            $AclDependencies = new AclDependencies();
            $acos = $AclDependencies->filterAcosForFrontend($acos->toArray());
            $this->set('acos', $acos);
            $this->viewBuilder()->setOption('serialize', ['acos']);
            return;
        }

        /** @var UsergroupsTable $UsergroupsTable */
        $UsergroupsTable = TableRegistry::getTableLocator()->get('Usergroups');
        $usergroup = $UsergroupsTable->newEmptyEntity();
        $usergroup = $UsergroupsTable->patchEntity($usergroup, $this->request->getData('Usergroup'));
        $UsergroupsTable->save($usergroup);

        if ($usergroup->hasErrors()) {
            //This throws the body content away :(
            $this->response = $this->response->withStatus(400);
            $this->set('error', $usergroup->getErrors());
            $this->viewBuilder()->setOption('serialize', ['error']);
            return;
        }

        //Save Acos
        /** @var ArosTable $ArosTable */
        $ArosTable = TableRegistry::getTableLocator()->get('Acl.Aros');
        /** @var Table $ArosAcos */
        //$ArosAcos = TableRegistry::getTableLocator()->get('ArosAcos');
        $aro = $ArosTable->find()
            ->where([
                'Aros.foreign_key' => $usergroup->get('id')
            ])
            ->first();


        $AclDependencies = new AclDependencies();
        $selectedAcos = $this->request->getData('Acos', []);
        $selectedAcos = $AclDependencies->getDependentAcos($AcosTable, $selectedAcos);

        // This is the CakePHP way, but it is super slow for many ACOs...
        //
        //$registry = new ComponentRegistry();
        //$Acl = new AclComponent($registry);
        //foreach ($selectedAcos as $acoId => $state) {
        //    if ($state === 1) {
        //        $Acl->allow($aro->get('id'), $acoId, '*');
        //    } else {
        //        $Acl->deny($aro->get('id'), $acoId, '*');
        //    }
        //}

        // Lightning fast workaround (we also done this in cake 2)
        // https://github.com/it-novum/openITCOCKPIT/blob/openITCOCKPIT-3.7.2/app/Controller/UsergroupsController.php#L213-L221
        /** @var ArosAcosTable $ArosAcosTable */
        $ArosAcosTable = TableRegistry::getTableLocator()->get('ArosAcos');

        $arosToAcos = [];
        foreach ($selectedAcos as $acoId => $state) {
            $arosToAcos[] = $ArosAcosTable->newEntity([
                'aro_id'  => $aro->get('id'),
                'aco_id'  => $acoId,
                '_create' => (int)($state === 1),
                '_read'   => (int)($state === 1),
                '_update' => (int)($state === 1),
                '_delete' => (int)($state === 1),
            ]);
        }
        $ArosAcosTable->saveMany($arosToAcos);

        Cache::clear('permissions');
        $this->set('usergroup', $usergroup);
        $this->viewBuilder()->setOption('serialize', ['usergroup']);
    }

    /**
     * @param int|null $id
     * @throws \Exception
     */
    public function edit($id = null) {
        if (!$this->isJsonRequest()) {
            $SystemConfig = new SystemConfig();
            $this->set('systemname', $SystemConfig->getSystemname());
            //Only ship html template
            return;
        }

        /** @var UsergroupsTable $UsergroupsTable */
        $UsergroupsTable = TableRegistry::getTableLocator()->get('Usergroups');

        if (!$UsergroupsTable->existsById($id)) {
            throw new NotFoundException(__('User group not found'));
        }

        $usergroup = $UsergroupsTable->find()
            ->contain([
                'Aros' => [
                    'Acos'
                ]
            ])
            ->where([
                'Usergroups.id' => $id
            ])
            ->firstOrFail();

        /** @var AcosTable $AcosTable */
        $AcosTable = TableRegistry::getTableLocator()->get('Acl.Acos');
        if ($this->request->is('get')) {
            $acos = $AcosTable->find('threaded')
                ->disableHydration()
                ->all();
            $AclDependencies = new AclDependencies();
            $acos = $AclDependencies->filterAcosForFrontend($acos->toArray());

            $this->set('usergroup', $usergroup);
            $this->set('acos', $acos);
            $this->viewBuilder()->setOption('serialize', ['usergroup', 'acos']);
            return;
        }

        if ($this->request->is('post')) {
            $usergroup = $UsergroupsTable->get($id);
            $usergroup->setAccess('id', false);
            if ($usergroup->get('name') === 'Administrator') {
                //An 'Administrator' is required!
                $usergroup->setAccess('name', false);
            }

            $usergroup = $UsergroupsTable->patchEntity($usergroup, $this->request->getData('Usergroup'));
            $UsergroupsTable->save($usergroup);

            if ($usergroup->hasErrors()) {
                $this->response = $this->response->withStatus(400);
                $this->set('error', $usergroup->getErrors());
                $this->viewBuilder()->setOption('serialize', ['error']);
                return;
            }

            //Save Acos
            $AclDependencies = new AclDependencies();
            $selectedAcos = $this->request->getData('Acos');
            $selectedAcos = $AclDependencies->getDependentAcos($AcosTable, $selectedAcos);

            /** @var ArosTable $ArosTable */
            $ArosTable = TableRegistry::getTableLocator()->get('Acl.Aros');
            /** @var ArosAcosTable c */
            $ArosAcosTable = TableRegistry::getTableLocator()->get('ArosAcos');
            $aro = $ArosTable->find()
                ->where([
                    'Aros.foreign_key' => $usergroup->get('id')
                ])
                ->firstOrFail();

            //Drop old permissions
            $ArosAcosTable->deleteAll([
                'ArosAcos.aro_id' => $aro->get('id')
            ]);

            $arosToAcos = [];
            foreach ($selectedAcos as $acoId => $state) {
                $arosToAcos[] = $ArosAcosTable->newEntity([
                    'aro_id'  => $aro->get('id'),
                    'aco_id'  => $acoId,
                    '_create' => (int)($state === 1),
                    '_read'   => (int)($state === 1),
                    '_update' => (int)($state === 1),
                    '_delete' => (int)($state === 1),
                ]);
            }

            $ArosAcosTable->saveMany($arosToAcos);
            Cache::clear('permissions');
            $this->set('usergroup', $usergroup);
            $this->viewBuilder()->setOption('serialize', ['usergroup']);
        }
    }

    /**
     * @param int|null $id
     */
    public function delete($id = null) {
        if (!$this->request->is('post')) {
            throw new MethodNotAllowedException();
        }

        /** @var UsergroupsTable $UsergroupsTable */
        $UsergroupsTable = TableRegistry::getTableLocator()->get('Usergroups');

        if (!$UsergroupsTable->existsById($id)) {
            throw new NotFoundException(__('User group not found'));
        }

        $usergroup = $UsergroupsTable->get($id, [
            'contain' => [
                'Users'
            ]
        ]);
        if (!empty($usergroup->get('users'))) {
            $usedBy = [
                [
                    'baseUrl'      => '#',
                    'state'        => 'UsersIndex',
                    'state_params' => [
                        'usergroup_id' => $id
                    ],
                    'message'      => __('Used by users'),
                    'module'       => 'Core'
                ]
            ];

            $this->response = $this->response->withStatus(400);
            $this->set('success', false);
            $this->set('id', $id);
            $this->set('message', __('Issue while deleting user role'));
            $this->set('usedBy', $usedBy);
            $this->viewBuilder()->setOption('serialize', ['success', 'id', 'message', 'usedBy']);
            return;
        }

        if ($UsergroupsTable->delete($usergroup)) {
            $this->set('success', true);
            $this->viewBuilder()->setOption('serialize', ['success']);
            return;
        }

        $this->response = $this->response->withStatus(500);
        $this->set('success', false);
        $this->viewBuilder()->setOption('serialize', ['success']);
    }
}
