<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Lib;


use itnovum\openITCOCKPIT\Core\PerfdataBackend;
use Symfony\Component\Yaml\Yaml;

class SystemConfig {

    /**
     * @var array|mixed
     */
    private $config = [];

    public function __construct() {
        $configFile = CONFIG . 'interface.yml';

        if (!file_exists($configFile)) {
            throw new \RuntimeException(sprintf('Configuration file %s is missing!', $configFile));
        }

        $config = [];
        if (file_exists($configFile)) {
            $YamlParser = new Yaml();
            $config = $YamlParser->parse(file_get_contents($configFile));
        }

        $this->config = $config;
    }

    /**
     * @return string
     */
    public function getSystemname() {
        return $this->config['systemname'] ?? 'openITCOCKPIT Satellite';
    }

    /**
     * @return string
     */
    public function getTitle() {
        return $this->config['title'] ?? 'Monitoring powered by openITCOCKPIT';
    }

    public function getMappingFile() {
        return $this->config['mapping_file'] ?? '/opt/openitc/nagios/etc/mapping.json';
    }

    /**
     * @return bool
     */
    public function getDisableLoginAnimation() {
        $value = $this->config['disable_login_animation'] ?? 0;
        $value = (int)$value;

        return (bool)$value;
    }

    public function getLoginLogoPath() {
        return $this->config['login_logo'] ?? '/img/logos/openitcockpit-logo-url-light.png';
    }

    /**
     * @return bool
     */
    public function getShowLinksInLogin() {
        $value = $this->config['show_link_in_login'] ?? 1;
        $value = (int)$value;

        return (bool)$value;
    }

    public function getSidebarLogoPath() {
        return $this->config['sidebar_logo'] ?? '/img/favicons/favicon-32x32.png';
    }

    /**
     * @return string
     */
    public function getWebserverUser() {
        return $this->config['webserver_user'] ?? 'www-data';
    }

    /**
     * @return string
     */
    public function getWebserverGroup() {
        return $this->config['webserver_group'] ?? 'www-data';
    }

    /**
     * @return int
     */
    public function getPreselectedDowntimetype() {
        return $this->config['preselected_downtime_type'] ?? 0; // 0 => Individual host ; 1 => Host including services
    }

    /**
     * @return PerfdataBackend
     */
    public function getPerfdataBackend() {
        $backend = $this->config['perfdata_backend'] ?? 'MySQL';

        return new PerfdataBackend($backend);
    }

    public function getAgeHostchecks() {
        return $this->config['age_hostchecks'] ?? 5;
    }

    public function getAgeHostAcknowledgements() {
        return $this->config['age_host_acknowledgements'] ?? 60;
    }

    public function getAgeHostNotifications() {
        return $this->config['age_host_notifications'] ?? 60;
    }

    public function getAgeHostStatehistory() {
        return $this->config['age_host_statehistory'] ?? 90;
    }

    public function getAgeHostDowntimes() {
        return $this->config['age_host_downtimes'] ?? 60;
    }

    public function getAgeServicechecks() {
        return $this->config['age_servicechecks'] ?? 5;
    }

    public function getAgeServiceAcknowledgements() {
        return $this->config['age_service_acknowledgements'] ?? 60;
    }

    public function getAgeServiceNotifications() {
        return $this->config['age_service_notifications'] ?? 60;
    }

    public function getAgeServiceStatehistory() {
        return $this->config['age_service_statehistory'] ?? 90;
    }

    public function getAgeServiceDowntimes() {
        return $this->config['age_service_downtimes'] ?? 60;
    }

    public function getAgeLogentries() {
        return $this->config['age_logentries'] ?? 5;
    }

    public function getAgePerfdata() {
        return $this->config['age_perfdata'] ?? 90;
    }

    public function getGraphitePrefix() {
        return $this->config['graphite_prefix'] ?? 'openitcockpit';
    }

    public function getGraphiteAddress() {
        return $this->config['graphite_address'] ?? 'localhost';
    }

    public function getGraphitePort() {
        return $this->config['graphite_port'] ?? 2003;
    }

    public function getGraphiteStorageSchema() {
        return $this->config['graphite_storage_schema'] ?? '60s:90d';
    }

    public function getGraphiteTimezone() {
        return $this->config['graphite_timezone'] ?? 'Europe/Berlin';
    }

    /**
     * @return int
     */
    public function getDockerAutoNetworkingEnabled() {
        $value = $this->config['docker_auto_networking'] ?? 1;
        return (int)$value;
    }

    public function getDockerComposeSubnet() {
        return $this->config['docker_compose_subnet'] ?? '';
    }

    public function getMapJsonFile(){
        return $this->config['maps_json'] ?? '/opt/openitc/nagios/etc/maps.json';
    }

    public function getMapZipFile(){
        return $this->config['maps_zip'] ?? '/opt/openitc/nagios/etc/maps.zip';
    }

}
