<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Model\Table;

use App\Model\Entity\Hostgroup;
use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * Hostgroups Model
 *
 * @property \App\Model\Table\HostgroupMembersTable&\Cake\ORM\Association\HasMany $HostgroupMembers
 *
 * @method \App\Model\Entity\Hostgroup newEmptyEntity()
 * @method \App\Model\Entity\Hostgroup newEntity(array $data, array $options = [])
 * @method \App\Model\Entity\Hostgroup[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Hostgroup get($primaryKey, $options = [])
 * @method \App\Model\Entity\Hostgroup findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \App\Model\Entity\Hostgroup patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Hostgroup[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \App\Model\Entity\Hostgroup|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Hostgroup saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Hostgroup[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Hostgroup[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \App\Model\Entity\Hostgroup[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Hostgroup[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class HostgroupsTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('hostgroups');
        $this->setDisplayField('id');
        $this->setPrimaryKey('id');

        $this->hasMany('HostgroupMembers', [
            'foreignKey' => 'hostgroup_id',
        ]);

    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        $validator
            ->integer('id')
            ->allowEmptyString('id', null, 'create');

        $validator
            ->scalar('uuid')
            ->maxLength('uuid', 37)
            ->requirePresence('uuid', 'create')
            ->notEmptyString('uuid')
            ->add('uuid', 'unique', ['rule' => 'validateUnique', 'provider' => 'table']);

        return $validator;
    }

    /**
     * Returns a rules checker object that will be used for validating
     * application integrity.
     *
     * @param \Cake\ORM\RulesChecker $rules The rules object to be modified.
     * @return \Cake\ORM\RulesChecker
     */
    public function buildRules(RulesChecker $rules): RulesChecker {
        $rules->add($rules->isUnique(['uuid']));

        return $rules;
    }


    /**
     * @param string $hostgroupUuid
     * @return Hostgroup
     */
    public function getHostsByHostgroupForMaps($hostgroupUuid) {
        $hostgroup = $this->find()
            ->select([
                'Hostgroups.id',
                'Hostgroups.uuid',
                'Hostgroups.description',
                'HostgroupMapping.display_name'
            ])
            ->contain([
                'HostgroupMembers' => function (Query $q) {
                    $q->select([
                        'HostgroupMembers.id',
                        'HostgroupMembers.hostgroup_id',
                        'HostgroupMembers.host_uuid',
                        'HostMapping.display_name'
                    ])
                        ->join([
                            [
                                'table'      => 'mapping',
                                'alias'      => 'HostMapping',
                                'type'       => 'INNER',
                                'conditions' => [
                                    'HostgroupMembers.host_uuid = HostMapping.uuid',
                                ],
                            ]
                        ]);
                    return $q;
                }
            ])
            ->join([
                [
                    'table'      => 'mapping',
                    'alias'      => 'HostgroupMapping',
                    'type'       => 'INNER',
                    'conditions' => [
                        'Hostgroups.uuid = HostgroupMapping.uuid',
                    ],
                ]
            ])
            ->where([
                'Hostgroups.uuid' => $hostgroupUuid
            ])
            ->disableHydration()
            ->first();

        return $hostgroup ?? [];
    }
}
