<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.
declare(strict_types=1);

namespace App\Model\Table;

use Cake\ORM\Query;
use Cake\ORM\RulesChecker;
use Cake\ORM\Table;
use Cake\Validation\Validator;

/**
 * Perfdata Model
 *
 * @method \App\Model\Entity\Perfdata newEmptyEntity()
 * @method \App\Model\Entity\Perfdata newEntity(array $data, array $options = [])
 * @method \App\Model\Entity\Perfdata[] newEntities(array $data, array $options = [])
 * @method \App\Model\Entity\Perfdata get($primaryKey, $options = [])
 * @method \App\Model\Entity\Perfdata findOrCreate($search, ?callable $callback = null, $options = [])
 * @method \App\Model\Entity\Perfdata patchEntity(\Cake\Datasource\EntityInterface $entity, array $data, array $options = [])
 * @method \App\Model\Entity\Perfdata[] patchEntities(iterable $entities, array $data, array $options = [])
 * @method \App\Model\Entity\Perfdata|false save(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Perfdata saveOrFail(\Cake\Datasource\EntityInterface $entity, $options = [])
 * @method \App\Model\Entity\Perfdata[]|\Cake\Datasource\ResultSetInterface|false saveMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Perfdata[]|\Cake\Datasource\ResultSetInterface saveManyOrFail(iterable $entities, $options = [])
 * @method \App\Model\Entity\Perfdata[]|\Cake\Datasource\ResultSetInterface|false deleteMany(iterable $entities, $options = [])
 * @method \App\Model\Entity\Perfdata[]|\Cake\Datasource\ResultSetInterface deleteManyOrFail(iterable $entities, $options = [])
 */
class PerfdataTable extends Table {
    /**
     * Initialize method
     *
     * @param array $config The configuration for the Table.
     * @return void
     */
    public function initialize(array $config): void {
        parent::initialize($config);

        $this->setTable('statusengine_perfdata');
    }

    /**
     * Default validation rules.
     *
     * @param \Cake\Validation\Validator $validator Validator instance.
     * @return \Cake\Validation\Validator
     */
    public function validationDefault(Validator $validator): Validator {
        //Readonly table
        return $validator;
    }

    /**
     * @param string $hostUuid
     * @param string $serviceUuid
     * @param string $label
     * @param int $startTimestamp
     * @param int $endTimestamp
     * @return array
     */
    public function getServicePerfdata(string $hostUuid, string $serviceUuid, string $label, int $startTimestamp, int $endTimestamp) {
        //dd(['start' => $startTimestamp, 'end' => $endTimestamp]);
        $query = $this->find()
            ->where([
                'hostname'            => $hostUuid,
                'service_description' => $serviceUuid,
                'label'               => $label,
                'timestamp_unix >='   => $startTimestamp,
                'timestamp_unix <='   => $endTimestamp
            ])
            ->order([
                'timestamp_unix' => 'ASC'
            ])
            ->disableHydration();

        $query->all();
        if ($query === null) {
            return [];
        }

        return $query->toArray();
    }

    /**
     * @param string $hostUuid
     * @param string $serviceUuid
     * @return array
     */
    public function findMetricsByUuid(string $hostUuid, string $serviceUuid) {
        $query = $this->find()
            ->select([
                'label'
            ])
            ->where([
                'hostname'            => $hostUuid,
                'service_description' => $serviceUuid
            ])
            ->disableHydration()
            ->group([
                'label'
            ])
            ->all();

        if ($query === null) {
            return [];
        }

        return $query->toArray();
    }
}
