<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace App\Policy;


use Acl\Controller\Component\AclComponent;
use Acl\Model\Table\ArosTable;
use App\Model\Table\UsergroupsTable;
use Authentication\Authenticator\UnauthenticatedException;
use Authorization\IdentityInterface;
use Authorization\Policy\RequestPolicyInterface;
use Cake\Controller\ComponentRegistry;
use Cake\Controller\Exception\SecurityException;
use Cake\Http\ServerRequest;
use Cake\ORM\TableRegistry;

class RequestPolicy implements RequestPolicyInterface {

    /**
     * Method to check if the request can be accessed
     *
     * @param IdentityInterface|null Identity
     * @param ServerRequest $request Server Request
     * @return bool
     */
    public function canAccess($identity, ServerRequest $request) {
        $controller = $request->getParam('controller');
        $action = $request->getParam('action');
        $plugin = $request->getParam('plugin'); //not used

        //FileDebugger::dump(sprintf('Controller: %s | Action: %s | Plugin: %s', $controller, $action, $plugin));

        if ($identity === null) {
            //User is not logged in!
            if (strtolower($controller) === 'users' && strtolower($action) === 'login') {
                return true;
            }

            throw new UnauthenticatedException();
        }

        // Load UsergroupsTable for Acl Plugin
        /** @var UsergroupsTable $UsergroupsTable */
        $UsergroupsTable = TableRegistry::getTableLocator()->get('Usergroups');

        $Collection = new ComponentRegistry();
        $Acl = new AclComponent($Collection);

        /** @var ArosTable $ArosTable */
        $ArosTable = TableRegistry::getTableLocator()->get('Acl.Aros');

        $usergroupId = $identity->get('usergroup_id');

        $usergroupHasAros = $ArosTable->exists([
            'Aros.foreign_key' => $usergroupId
        ]);

        if ($usergroupHasAros === false) {
            throw new SecurityException('No Aros defined for given usergroup_id!');
        }

        // Uncomment to disable ACL permission checks
        // return true;

        if ($plugin !== null) {
            return $Acl->check(['Usergroups' => ['id' => $usergroupId]], "$plugin/$controller/$action");
        }

        //Core request
        //dd($Acl->check(['Usergroups' => ['id' => $usergroupId]], "$controller/$action"));
        return $Acl->check(['Usergroups' => ['id' => $usergroupId]], "$controller/$action");
    }
}
