<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace itnovum\openITCOCKPIT\Core\Menu;


use App\Lib\PluginManager;

class Menu {

    const MENU_OVERVIEW = 'overview';
    const MENU_MONITORING = 'monitoring';
    const MENU_ADMINISTRATION = 'administration';
    const MENU_CONFIGURATION = 'configuration';

    /**
     * @var MenuHeadline[]
     */
    private $headlines = [];

    /**
     * @var array
     */
    private $PERMISSIONS = [];

    /**
     * Menu constructor.
     * @param array $PERMISSIONS
     */
    public function __construct($PERMISSIONS = []) {
        $this->PERMISSIONS = $PERMISSIONS;

        $Overview = new MenuHeadline(self::MENU_OVERVIEW, __('Overview'), 1);
        $Overview
            ->addLink(new MenuLink(
                __('Dashboards'),
                'DashboardsIndex',
                'Dashboard',
                'index',
                '',
                'fas fa-tachometer-alt',
                ['dashboard'],
                1
            ))
            ->addCategory(
                (new MenuCategory(
                    'maps_category',
                    __('Maps'),
                    3,
                    'fa fa-map-marker'
                ))
                    ->addLink(new MenuLink(
                        __('Auto Map'),
                        'AutomapsIndex',
                        'Services',
                        'automap',
                        '',
                        'fa-solid fa-wand-magic-sparkles',
                        ['automaps', 'auto', 'map'],
                        2
                    ))
            );

        $Monitoring = new MenuHeadline(self::MENU_MONITORING, __('Monitoring'), 2);
        $Monitoring
            ->addLink(new MenuLink(
                __('Hosts'),
                'HostsIndex',
                'Hosts',
                'index',
                '',
                'fa fa-desktop',
                ['hosts'],
                1
            ))
            ->addLink(new MenuLink(
                __('Services'),
                'ServicesIndex',
                'Services',
                'index',
                '',
                'fa fa-cogs',
                ['hosts'],
                2
            ));

        $Administration = new MenuHeadline(self::MENU_ADMINISTRATION, __('Administration'), 3);
        $Administration
            ->addCategory((new MenuCategory(
                'user_mgmt_category',
                __('User management'),
                1,
                'fa fa-users'
            ))
                ->addLink(new MenuLink(
                    __('Manage Users'),
                    'UsersIndex',
                    'users',
                    'index',
                    '',
                    'fa fa-user',
                    [],
                    1
                ))
                ->addLink(new MenuLink(
                    __('Manage User Roles'),
                    'UsergroupsIndex',
                    'usergroups',
                    'index',
                    '',
                    'fa fa-users',
                    [],
                    2
                ))
            )
            ->addCategory((new MenuCategory(
                'agent_category',
                __('openITCOCKPIT Agent'),
                2,
                'fa fa-user-secret'
            ))
                ->addLink(new MenuLink(
                    __('Agents Overview'),
                    'AgentconnectorsPull',
                    'agentconnector',
                    'overview',
                    '',
                    'fa fa-user-secret',
                    [],
                    1
                ))
            );


        $this->addHeadline($Overview);
        $this->addHeadline($Monitoring);
        $this->addHeadline($Administration);
    }

    /**
     * @param MenuHeadline $MenuHeadline
     * @return $this
     */
    public function addHeadline(MenuHeadline $MenuHeadline) {
        $headlines = $this->headlines;
        $headlines[] = $MenuHeadline;

        $this->headlines = [];

        //Merge headlines with the same name together
        $names = [];
        $headlinesToRemove = [];
        foreach ($headlines as $index => $headline) {
            /** @var MenuHeadline $headline */
            if (isset($names[$headline->getName()])) {
                // Merge with existing headline
                /** @var MenuHeadline $targetHeadline */
                $targetHeadline = $names[$headline->getName()];
                $headlinesToRemove = [$index];
                foreach ($headline->getItems() as $item) {
                    if ($item instanceof MenuLink) {
                        $targetHeadline->addLink($item);
                    }
                    if ($item instanceof MenuCategory) {
                        $targetHeadline->addCategory($item);
                    }
                }
            } else {
                //Save name
                $names[$headline->getName()] = $headline;
            }
        }

        //Drop merged headlines to remove duplicates
        foreach ($headlinesToRemove as $index) {
            unset($headlines[$index]);
        }


        //Order headlines
        $indexesToOrder = [];
        foreach ($headlines as $index => $headline) {
            /** @var MenuHeadline $headline */
            $indexesToOrder[$index] = $headline->getOrder();
        }

        asort($indexesToOrder);

        foreach ($indexesToOrder as $index => $orderNumber) {
            $this->headlines[] = $headlines[$index];
        }

        return $this;
    }

    /**
     * @return array
     */
    public function getAllMenuCategories() {
        $menuCategories = [];
        foreach ($this->headlines as $headline) {
            /** @var MenuHeadline $headline */
            if (!isset($menuCategories[$headline->getName()])) {
                $menuCategories[$headline->getName()] = [];
            }

            foreach ($headline->getItems() as $item) {
                if ($item instanceof MenuCategory) {
                    $menuCategories[$headline->getName()][] = $item->getName();
                }
            }
        }

        return $menuCategories;
    }

    /**
     * @return MenuHeadline[]
     */
    public function getMenuItems() {
        $modules = PluginManager::getAvailablePlugins();
        foreach ($modules as $module) {
            $className = sprintf('\\%s\\Lib\\Menu', $module);
            if (class_exists($className)) {

                /** @var MenuInterface $PluginMenu */
                $PluginMenu = new $className();

                foreach ($PluginMenu->getHeadlines() as $headline) {
                    /** @var MenuHeadline $headline */
                    $this->addHeadline($headline);
                }
            }
        }


        $this->filterMenuByPermissions();
        $this->remoteEmptyMenuItems();

        return $this->headlines;
    }

    private function filterMenuByPermissions() {
        foreach ($this->headlines as $headline) {
            foreach ($headline->getItems() as $index => $item) {
                if ($item instanceof MenuCategory) {
                    /** @var MenuCategory $item */
                    foreach ($item->getLinks() as $subIndex => $MenuLink) {
                        if ($this->hasPermissions($MenuLink) === false) {
                            $item->removeMenuLinkByIndex($subIndex);
                        }
                    }

                } else {
                    /** @var MenuLink $item */
                    if ($this->hasPermissions($item) === false) {
                        $headline->removeMenuLinkByIndex($index);
                    }

                }
            }
        }
    }

    /**
     * @param MenuLink $MenuLink
     * @return bool
     */
    private function hasPermissions(MenuLink $MenuLink) {
        $plugin = $MenuLink->getLowerPlugin();
        $controller = $MenuLink->getLowerController();
        $action = $MenuLink->getLowerAction();

        if ($MenuLink->isModule()) {
            return isset($this->PERMISSIONS[$plugin][$controller][$action]);
        }

        return isset($this->PERMISSIONS[$controller][$action]);
    }

    private function remoteEmptyMenuItems() {
        foreach ($this->headlines as $headlineIndex => $headline) {
            foreach ($headline->getItems() as $index => $item) {
                if ($item instanceof MenuCategory) {
                    /** @var MenuCategory $item */
                    if ($item->hasLinks() === false) {
                        $headline->removeMenuCategoryByIndex($index);
                    }
                }
            }

            if ($headline->hasItems() === false) {
                unset($this->headlines[$headlineIndex]);
            }
        }

    }

}
