<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace itnovum\openITCOCKPIT\Core\ValueObjects;


use itnovum\openITCOCKPIT\Exceptions\UnknownStateException;

class ServiceStates {

    /**
     * @var array
     */
    private $stateNames = [
        0 => 'ok',
        1 => 'warning',
        2 => 'critical',
        3 => 'unknown'
    ];

    /**
     * @var array
     */
    private $stateIds = [
        'ok'       => 0,
        'warning'  => 1,
        'critical' => 2,
        'unknown'  => 3
    ];

    /**
     * @var array
     */
    private $states = [
        'ok'       => false,
        'warning'  => false,
        'critical' => false,
        'unknown'  => false
    ];

    /**
     * @param int $state
     * @param bool $value
     */
    public function setState($state, $value) {
        $key = $this->getStateNameByState($state);
        $this->states[$key] = $value;
    }

    /**
     * @return bool
     */
    public function hasOk() {
        return $this->states['ok'];
    }

    /**
     * @return bool
     */
    public function hasWarning() {
        return $this->states['warning'];
    }

    /**
     * @return bool
     */
    public function hasCritical() {
        return $this->states['critical'];
    }

    /**
     * @return bool
     */
    public function hasUnknown() {
        return $this->states['unknown'];
    }


    /**
     * @return array
     */
    public function asArray() {
        return $this->states;
    }


    /**
     * @return array
     */
    public function asIntegerArray() {
        $states = [];
        if ($this->hasOk()) {
            $states[] = 0;
        }
        if ($this->hasWarning()) {
            $states[] = 1;
        }
        if ($this->hasCritical()) {
            $states[] = 2;
        }
        if ($this->hasUnknown()) {
            $states[] = 3;
        }
        return $states;
    }

    /**
     * @return array
     */
    public function getAvailableStateIds() {
        return $this->stateIds;
    }

    /**
     * @param int $state
     * @return string
     * @throws UnknownStateException
     */
    public function getStateNameByState($state) {
        if (isset($this->stateNames[$state])) {
            return $this->stateNames[$state];
        }

        throw new UnknownStateException(sprintf('State "%s" is not a valid state id', $state));
    }

}
