<?php
// Copyright (C) <2015-present>  <it-novum GmbH>
//
// This file is licensed under the terms of the openITCOCKPIT Enterprise Edition license agreement.
// The license agreement and license key were sent with the order confirmation.

namespace itnovum\openITCOCKPIT\Core\ValueObjects;


use itnovum\openITCOCKPIT\Exceptions\UnknownStateException;

class StateTypes {

    /**
     * @var array
     */
    private $stateTypeNames = [
        0 => 'soft',
        1 => 'hard'
    ];

    /**
     * @var array
     */
    private $stateTypeIds = [
        'soft' => 0,
        'hard' => 1
    ];

    /**
     * @var array
     */
    private $stateTypes = [
        'soft' => false,
        'hard' => false
    ];

    /**
     * @param int $state
     * @param bool $value
     */
    public function setStateType($stateType, $value) {
        $key = $this->getStateTypeNameByStateType($stateType);
        $this->stateTypes[$key] = $value;
    }

    /**
     * @return bool
     */
    public function hasSoft() {
        return $this->stateTypes['soft'];
    }

    /**
     * @return bool
     */
    public function hasHard() {
        return $this->stateTypes['hard'];
    }

    /**
     * @return array
     */
    public function asArray() {
        return $this->stateTypes;
    }

    /**
     * @return array
     */
    public function asIntegerArray() {
        $stateTypes = [];
        if ($this->hasSoft()) {
            $stateTypes[] = 0;
        }
        if ($this->hasHard()) {
            $stateTypes[] = 1;
        }
        return $stateTypes;
    }

    /**
     * @return array
     */
    public function getAvailableStateTypeIds() {
        return $this->stateTypeIds;
    }

    /**
     * @param int $stateType
     * @return string
     * @throws UnknownStateException
     */
    public function getStateTypeNameByStateType($stateType) {
        if (isset($this->stateTypeNames[$stateType])) {
            return $this->stateTypeNames[$stateType];
        }

        throw new UnknownStateException(sprintf('State type "%s" is not a valid state type id', $stateType));
    }

}
