angular.module('openITCOCKPIT')
    .controller('HostsBrowserController', function($scope, $http, $stateParams, $interval, SortService){

        $scope.currentPage = 1;
        $scope.selectedTab = 'tab1';
        $scope.useScroll = true;
        SortService.setSort('Servicestatus.current_state');
        SortService.setDirection('desc');

        /** Filter settings **/
        var defaultFilter = function(){
            $scope.stateOk = true;
            $scope.stateWarning = true;
            $scope.stateCritical = true;
            $scope.stateUnknown = true;
            $scope.current_state = [];//
            $scope.filter_service_name = '';
            $scope.filter_output = '';
            $scope.showFilter = false;

            $scope.filter = {
                Servicestatus: {
                    acknowledged: false,
                    not_acknowledged: false,
                    in_downtime: false,
                    not_in_downtime: false
                }
            };
        };

        defaultFilter();
        /** End filter Settings **/

        $scope.resetFilter = function(){
            defaultFilter();
        };

        $scope.init = true;

        $scope.showFlashMsg = function(){
            new Noty({
                theme: 'metroui',
                type: 'success',
                layout: 'topCenter',
                text: $scope.flashMshStr,
                timeout: 4000
            }).show();

            $scope.showFlashSuccess = true;
            $scope.autoRefreshCounter = 5;
            var interval = $interval(function(){
                $scope.autoRefreshCounter--;
                if($scope.autoRefreshCounter === 0){
                    $scope.load();
                    $interval.cancel(interval);
                    $scope.showFlashSuccess = false;
                }
            }, 1000);
        };

        $scope.load = function(){
            $http.get("/hosts/browser.json", {
                params: {
                    uuid: $stateParams.uuid
                }
            }).then(function(result){
                $scope.hoststatus = result.data.hoststatus;
                $scope.acknowledgement = result.data.acknowledgement;
                $scope.downtime = result.data.downtime;

                $scope.init = false;
                $scope.loadServices(); //Fires on load of page
            });
        };

        $scope.getObjectsForExternalCommand = function(){
            return [{
                Host: {
                    uuid: $scope.hoststatus.hostname
                }
            }];
        };

        $scope.getObjectsForDowntimeDelete = function(){
            return [{
                Host: {
                    uuid: $scope.hoststatus.hostname
                },
                Downtime: {
                    internal_downtime_id: $scope.downtime.internal_downtime_id
                }
            }];
        };

        $scope.loadServices = function(){
            var hasBeenAcknowledged = '';
            var inDowntime = '';
            if($scope.filter.Servicestatus.acknowledged ^ $scope.filter.Servicestatus.not_acknowledged){
                hasBeenAcknowledged = $scope.filter.Servicestatus.acknowledged === true;
            }
            if($scope.filter.Servicestatus.in_downtime ^ $scope.filter.Servicestatus.not_in_downtime){
                inDowntime = $scope.filter.Servicestatus.in_downtime === true;
            }

            $http.get("/services/index.json", {
                params: {
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'filter[Servicestatus.hostname]': $stateParams.uuid,

                    'filter[Servicestatus.current_state][]': $scope.current_state,
                    'filter[HostMapping.display_name]': $scope.filter_host_name,
                    'filter[ServiceMapping.display_name]': $scope.filter_service_name,
                    'filter[Servicestatus.output]': $scope.filter_output,
                    'filter[Servicestatus.problem_has_been_acknowledged]': hasBeenAcknowledged,
                    'filter[Servicestatus.scheduled_downtime_depth]': inDowntime
                }
            }).then(function(result){
                $scope.servicestatus = result.data.servicestatus;
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;
                $scope.init = false;
            });
        };

        $scope.loadTimezone = function(){
            $http.get("/angular/user_timezone.json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.timezone = result.data.timezone;
            });
        };

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        $scope.getColorOfState = function(state){
            if(state === 0){
                return 'up';
            }
            if(state === 1){
                return 'down';
            }
            if(state === 2){
                return 'unreachable';
            }
        };

        $scope.$watchGroup(['stateOk', 'stateWarning', 'stateCritical', 'stateUnknown', 'filter_service_name', 'filter_output'], function(){
            if($scope.init){
                return;
            }
            $scope.current_state = [];
            if($scope.stateOk){
                $scope.current_state.push('ok');
            }
            if($scope.stateWarning){
                $scope.current_state.push('warning');
            }
            if($scope.stateCritical){
                $scope.current_state.push('critical');
            }
            if($scope.stateUnknown){
                $scope.current_state.push('unknown');
            }
            //Load on filter
            $scope.loadServices();
        });

        $scope.$watch('filter', function(){
            if($scope.init){
                return;
            }

            $scope.loadServices();
        }, true);

        //Sort/direction reload
        SortService.setCallback($scope.loadServices);
        SortService.triggerReload();


        //Fire on page load
        $scope.load();
        $scope.loadTimezone();

        if($("[rel=tooltip]").length){
            $("[rel=tooltip]").tooltip();
        }

        $scope.stateIsUp = function(){
            return parseInt($scope.hoststatus.current_state, 10) === 0;
        };

        $scope.stateIsDown = function(){
            return parseInt($scope.hoststatus.current_state, 10) === 1;
        };

        $scope.stateIsUnreachable = function(){
            return parseInt($scope.hoststatus.current_state, 10) === 2;
        };

        $scope.stateIsNotInMonitoring = function(){
            // return !$scope.hoststatus.isInMonitoring;
            return temporaryAlwaysInMonitoring = false;
        };

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };
    });
