angular.module('openITCOCKPIT')
    .controller('ServicesIndexController', function($scope, $http, $interval, $stateParams, SortService, MassChangeService){

        $scope.currentPage = 1;
        $scope.useScroll = true;

        SortService.setSort('Servicestatus.current_state');
        SortService.setDirection('desc');

        /** Filter settings **/
        var defaultFilter = function(){
            $scope.stateOk = true;
            $scope.stateWarning = true;
            $scope.stateCritical = true;
            $scope.stateUnknown = true;
            $scope.current_state = [];//
            $scope.filter_host_name = '';
            $scope.filter_service_name = $stateParams.servicename || '';
            $scope.filter_output = '';
            $scope.showFilter = false;

            $scope.filter = {
                Servicestatus: {
                    acknowledged: false,
                    not_acknowledged: false,
                    in_downtime: false,
                    not_in_downtime: false
                }
            };
        };

        defaultFilter();
        /** End filter Settings **/

        $scope.massChange = {};
        $scope.init = true;

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
        };

        $scope.showFlashMsg = function(){
            new Noty({
                theme: 'metroui',
                type: 'success',
                layout: 'topCenter',
                text: $scope.flashMshStr,
                timeout: 4000
            }).show();

            $scope.showFlashSuccess = true;
            $scope.autoRefreshCounter = 5;
            var interval = $interval(function(){
                $scope.autoRefreshCounter--;
                if($scope.autoRefreshCounter === 0){
                    $scope.load();
                    $interval.cancel(interval);
                    $scope.showFlashSuccess = false;
                }
            }, 1000);
        };

        $scope.load = function(){
            var hasBeenAcknowledged = '';
            var inDowntime = '';
            if($scope.filter.Servicestatus.acknowledged ^ $scope.filter.Servicestatus.not_acknowledged){
                hasBeenAcknowledged = $scope.filter.Servicestatus.acknowledged === true;
            }
            if($scope.filter.Servicestatus.in_downtime ^ $scope.filter.Servicestatus.not_in_downtime){
                inDowntime = $scope.filter.Servicestatus.in_downtime === true;
            }

            $http.get("/services/index.json", {
                params: {
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'filter[Servicestatus.current_state][]': $scope.current_state,
                    'filter[HostMapping.display_name]': $scope.filter_host_name,
                    'filter[ServiceMapping.display_name]': $scope.filter_service_name,
                    'filter[Servicestatus.output]': $scope.filter_output,
                    'filter[Servicestatus.problem_has_been_acknowledged]': hasBeenAcknowledged,
                    'filter[Servicestatus.scheduled_downtime_depth]': inDowntime
                }
            }).then(function(result){
                $scope.servicestatus = result.data.servicestatus;
                $scope.paging = result.data.paging;
                $scope.scroll = result.data.scroll;
                $scope.init = false;
            });

        };

        $scope.loadTimezone = function(){
            $http.get("/angular/user_timezone.json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.timezone = result.data.timezone;
            });
        };

        $scope.parseUrlFilter = function(){
            if($stateParams.statename){
                $scope.stateOk = false;
                $scope.stateWarning = false;
                $scope.stateCritical = false;
                $scope.stateUnknown = false;
                switch($stateParams.statename){
                    case 'ok':
                        $scope.stateOk = true;
                        break;
                    case 'warning':
                        $scope.stateWarning = true;
                        break;
                    case 'critical':
                        $scope.stateCritical = true;
                        break;
                    default:
                        $stateParams.statename = 'unknown';
                        $scope.stateUnknown = true;
                        break;
                }
                $scope.current_state = [$stateParams.statename];
            }

            if($stateParams.hostdisplayname){
                $scope.filter_host_name = decodeURI($stateParams.hostdisplayname);
            }
        };

        $scope.changepage = function(page){
            $scope.undoSelection();
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        $scope.selectAll = function(){
            if($scope.servicestatus){
                for(var key in $scope.servicestatus){
                    var serviceUuid = $scope.servicestatus[key].service_description;
                    $scope.massChange[serviceUuid] = true;
                }
            }
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        $scope.getObjectsForExternalCommand = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.servicestatus){
                for(var serviceUuid in selectedObjects){
                    if(serviceUuid === $scope.servicestatus[key].service_description){
                        objects[serviceUuid] = {
                            Host: {
                                uuid: $scope.servicestatus[key].hostname
                            },
                            Service: {
                                uuid: $scope.servicestatus[key].service_description
                            }
                        }
                    }
                }
            }
            return objects;
        };

        // Fire on page load
        $scope.loadTimezone();
        $scope.$watchGroup(['stateOk', 'stateWarning', 'stateCritical', 'stateUnknown', 'filter_host_name', 'filter_service_name', 'filter_output'], function(){
            if($scope.init){
                return;
            }
            $scope.current_state = [];
            $scope.undoSelection();
            if($scope.stateOk){
                $scope.current_state.push('ok');
            }
            if($scope.stateWarning){
                $scope.current_state.push('warning');
            }
            if($scope.stateCritical){
                $scope.current_state.push('critical');
            }
            if($scope.stateUnknown){
                $scope.current_state.push('unknown');
            }
            $scope.load();
        });

        $scope.$watch('filter', function(){
            if($scope.init){
                return;
            }

            $scope.load();
        }, true);

        $scope.parseUrlFilter();
        SortService.setCallback($scope.load);
        SortService.triggerReload();

        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);

    });
