angular.module('openITCOCKPIT')
    .controller('UsersIndexController', function($scope, $http, SortService, MassChangeService, $stateParams){

        $scope.currentPage = 1;
        $scope.useScroll = false;
        $scope.deleteUrl = '/users/delete/';
        $scope.massChange = {};

        SortService.setSort('Users.username');
        SortService.setDirection('asc');
        $scope.usergroup_id = $stateParams.usergroup_id;

        /*** Filter Settings ***/
        var defaultFilter = function(){
            $scope.filter = {
                Users: {
                    username: '',
                    firstname: '',
                    lastname: '',
                    usergroup_id: ($scope.usergroup_id) ? $scope.usergroup_id : []
                }
            };
        };
        /** Filter Settings **/

        $scope.triggerFilter = function(){
            $scope.showFilter = !$scope.showFilter === true;
        };

        $scope.resetFilter = function(){
            defaultFilter();
        };

        $scope.init = true;
        $scope.load = function(){
            $http.get("/Users/index.json", {
                params: {
                    'scroll': $scope.useScroll,
                    'sort': SortService.getSort(),
                    'page': $scope.currentPage,
                    'direction': SortService.getDirection(),
                    'filter[Users.username]': $scope.filter.Users.username,
                    'filter[Users.firstname]': $scope.filter.Users.firstname,
                    'filter[Users.lastname]': $scope.filter.Users.lastname,
                    'filter[Users.usergroup_id][]': $scope.filter.Users.usergroup_id
                }
            }).then(function(result){
                $scope.users = result.data.users;
                $scope.myUserId = result.data.myUserId;
                $scope.scroll = result.data.scroll;
                $scope.paging = result.data.paging;
                $scope.init = false;
            });
        };

        $scope.loadUsergroups = function(){
            $http.get("/users/loadUsergroups.json", {
                params: {
                    'angular': true
                }
            }).then(function(result){
                $scope.usergroups = result.data.usergroups;
            }, function errorCallback(result){
                if(result.status === 403){
                    $state.go('403');
                }

                if(result.status === 404){
                    $state.go('404');
                }
            });
        };


        $scope.getObjectForDelete = function(user){
            var object = {};
            object[user.id] = user.username;
            return object;
        };

        $scope.getObjectsForDelete = function(){
            var objects = {};
            var selectedObjects = MassChangeService.getSelected();
            for(var key in $scope.users){
                for(var id in selectedObjects){
                    if(id == $scope.users[key].id){
                        objects[id] = $scope.users[key].username;
                    }
                }
            }
            return objects;
        };

        $scope.selectAll = function(){
            if($scope.users){
                for(var key in $scope.users){
                    var id = $scope.users[key].id;
                    if(id !== $scope.myUserId){
                        $scope.massChange[id] = true;
                    }
                }
            }
        };

        $scope.undoSelection = function(){
            MassChangeService.clearSelection();
            $scope.massChange = MassChangeService.getSelected();
            $scope.selectedElements = MassChangeService.getCount();
        };

        //Fire on page load
        defaultFilter();
        $scope.load();
        $scope.loadUsergroups();

        $scope.$watch('filter', function(){
            $scope.currentPage = 1;
            $scope.load();
        }, true);

        $scope.changepage = function(page){
            if(page !== $scope.currentPage){
                $scope.currentPage = page;
                $scope.load();
            }
        };

        $scope.changeMode = function(val){
            $scope.useScroll = val;
            $scope.load();
        };

        SortService.setCallback($scope.load);
        SortService.triggerReload();

        $scope.$watch('massChange', function(){
            MassChangeService.setSelected($scope.massChange);
            $scope.selectedElements = MassChangeService.getCount();
        }, true);

    });
