<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Converter;

class Byte {

    /**
     * @param int|float $value in bytes
     * @return string
     */
    public static function asHuman($value) {
        $units = [
            'B',
            'kB',
            'MB',
            'GB',
            'TB',
            'EB',
            'ZB',
            'YB'
        ];

        $i = 0;
        while ($value > 1000) {
            $value = $value / 1024;
            $i++;
        }

        return round($value, 2) . $units[$i];
    }

    public static function supportedUnits() {
        return ['B', 'kB', 'KiB', 'MB', 'MiB', 'GB', 'GiB', 'TB', 'TiB', 'PB', 'PiB', 'EB', 'EiB'];
    }

    public static function scaleBytesToUnit($bytes, $targetUnit = 'B') {
        $binaryUnits = ['KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB']; // IEC units (1024)
        $decimalUnits = ['kB', 'MB', 'GB', 'TB', 'PB', 'EB']; // SI unit (1000)

        if ($targetUnit === 'B') {
            return $bytes;
        }

        if (in_array($targetUnit, $binaryUnits, true)) {
            // Convert bytes into IEC unit
            foreach ($binaryUnits as $unit) {
                if ($unit !== $targetUnit) {
                    $bytes = $bytes / 1024;
                }
            }
            return $bytes;
        }

        // Convert bytes into SI unit
        foreach ($decimalUnits as $unit) {
            if ($unit !== $targetUnit) {
                $bytes = $bytes / 1000;
            }
        }
        return $bytes;
    }

    /**
     * Converts 5872025.6 bytes/s to 5.6MB/s
     *
     * @param int|float $bytes
     * @return string
     */
    public static function scaleBytesPerSecondForHumans($bytes) {
        $units = [
            'bytes/s',
            'kb/s',
            'MB/s',
            'GB/s',
            'TB/s',
            'EB/s'
        ];

        $i = 0;
        while ($bytes > 1024) {
            $bytes = $bytes / 1024;
            $i++;
        }

        return round($bytes, 2) . $units[$i];
    }

    /**
     * @param int|float $bytes
     * @return array
     */
    public static function scaleBytesPerSecondToBitsPerSecondForHumans($bytes) {
        $bits = $bytes * 8; // convert bytes to bits
        $kbits = $bits / 1024;
        $mbits = $kbits / 1024;
        $gbits = $mbits / 1024;

        if ($gbits >= 1) {
            return [
                'value'     => round($gbits, 2),
                'unit'      => 'Gbit/s ',
                'perf_unit' => 'gbits'
            ];
        }

        if ($mbits >= 1) {
            return [
                'value'     => round($mbits, 2),
                'unit'      => 'Mbit/s',
                'perf_unit' => 'mbits'
            ];
        }

        return [
            'value'     => round($kbits, 2),
            'unit'      => 'kbit/s',
            'perf_unit' => 'kbits'
        ];
    }

    /**
     * @param int $mbits value in Mbit/s
     * @return string Mbit/s or Gbit/s with unit
     */
    public static function scaleMbits($mbits) {
        if ($mbits >= 1000) {
            return $mbits / 1000 . ' Gbit/s';
        }

        return $mbits . ' Mbit/s';
    }
}
