<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;

/**
 * Class Customcheck
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class Customcheck extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('customchecks')) {
            return;
        }

        $requiredKeys = [
            'customchecks'
        ];

        foreach ($requiredKeys as $requiredKey) {
            if (!isset($this->json[$requiredKey])) {
                $this->setOutput(sprintf('Key "%s" not found in Agent output', $requiredKey));
                $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                return;
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $name = null;
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $name = trim($arg);
                    break;
            }
        }

        if (!isset($this->json['customchecks'][$name])) {
            $this->setOutput(sprintf('Custom check "%s" not found in Agent output', $name));
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $customcheck = $this->json['customchecks'][$name];
        $rc = intval($customcheck['rc']);
        if ($rc > 3) {
            $rc = 3;
        }

        $this->setStatuscode($rc);
        $output = explode("\n", $customcheck['stdout'], 2);

        $this->setOutput($output[0]);
        if (!empty($output[1])) {
            $this->setLongOutput($output[1]);
        }
    }
}
