<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;

use itnovum\openITCOCKPIT\Checks\Receiver\Converter\Byte;
use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class DockerContainerMemory
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class DockerContainerMemory extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('docker')) {
            return;
        }

        $requiredKeys = [
            'docker'
        ];

        foreach ($requiredKeys as $requiredKey) {
            if (!isset($this->json[$requiredKey])) {
                $this->setOutput(sprintf('Key "%s" not found in Agent output', $requiredKey));
                $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                return;
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $type = 'name'; //identifier 'id' or 'name' (name is default)
        $match = null;  // id or name of the docker container e.g. a652cff1d4
        $asPercentage = false;
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    if (trim($arg) === 'id') {
                        $type = 'id';
                    }
                    break;
                case 1:
                    $match = trim($arg);
                    break;
                case 2:
                    $warning = $arg;
                    break;
                case 3:
                    $critical = $arg;
                    break;
                case 4:
                    $asPercentage = $arg == 1;
                    break;
            }
        }

        foreach ($this->json['docker'] as $docker) {
            if (isset($docker['id']) && isset($docker['name']) && isset($docker['state'])) {
                if ($docker[$type] === $match) {
                    $this->setStatuscode(NagiosService::STATUS_OK);
                    $this->setOutput(sprintf(
                        'Memory usage is %s (%s%%) for container [%s] %s',
                        Byte::asHuman($docker['memory_used']),
                        round($docker['memory_percentage'], 2),
                        $docker['id'],
                        $docker['name']
                    ));

                    $this->addPerfdata(new PerfdataGauge(
                        'memory_usage_bytes',
                        $docker['memory_used'],
                        'B',
                        !$asPercentage ? ($warning * 1024 * 1024) : null,
                        !$asPercentage ? ($critical * 1024 * 1024) : null
                    ));

                    $this->addPerfdata(new PerfdataGauge(
                        'memory_usage',
                        round($docker['memory_percentage'], 2),
                        '%',
                        $asPercentage ? $warning : null,
                        $asPercentage ? $critical : null,
                        0,
                        100
                    ));


                    $usedMemory = $docker['memory_used'] / 1024 / 1024; // Warning and critical are in MB !
                    if ($asPercentage) {
                        $usedMemory = $docker['memory_percentage'];
                    }

                    if ($usedMemory >= $critical) {
                        $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                        return;
                    }

                    if ($usedMemory >= $warning) {
                        $this->setStatuscode(NagiosService::STATUS_WARNING);
                        return;
                    }
                    return;
                }
            }
        }

        $this->setOutput(sprintf('Container "%s" not found!', $match));
        $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
    }
}
