<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;

/**
 * Class DockerContainerRunning
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class DockerContainerRunning extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('docker')) {
            return;
        }

        $requiredKeys = [
            'docker'
        ];

        foreach ($requiredKeys as $requiredKey) {
            if (!isset($this->json[$requiredKey])) {
                $this->setOutput(sprintf('Key "%s" not found in Agent output', $requiredKey));
                $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                return;
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $type = 'name'; //identifier 'id' or 'name' (name is default)
        $match = null;  // id or name of the docker container e.g. a652cff1d4
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    if (trim($arg) === 'id') {
                        $type = 'id';
                    }
                    break;
                case 1:
                    $match = trim($arg);
                    break;
            }
        }

        foreach ($this->json['docker'] as $docker) {
            if (isset($docker['id']) && isset($docker['name']) && isset($docker['state'])) {
                if ($docker[$type] === $match) {
                    if ($docker['state'] === 'running') {
                        $this->setOutput(sprintf('Container [%s] %s is running. Status: %s', $docker['id'], $docker['name'], $docker['status']));
                        $this->setStatuscode(NagiosService::STATUS_OK);
                        return;
                    } else {
                        $this->setOutput(sprintf('Container [%s] %s is %s. Status: %s', $docker['id'], $docker['name'], $docker['state'], $docker['status']));
                        $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                        return;
                    }
                }
            }
        }

        $this->setOutput(sprintf('Container "%s" not found!', $match));
        $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
    }
}
