<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\Converter\Byte;
use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;

/**
 * Class NetStats
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class NetStats extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('net_stats')) {
            return;
        }

        $requiredKeys = [
            'net_stats'
        ];

        foreach ($requiredKeys as $requiredKey) {
            if (!isset($this->json[$requiredKey])) {
                $this->setOutput(sprintf('Key "%s" not found in Agent output', $requiredKey));
                $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                return;
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $downState = $arg;
                    break;
                case 1:
                    $device = $arg;
                    break;
            }
        }

        if (!isset($this->json['net_stats'][$device])) {
            $this->setOutput(sprintf('Network device "%s" not found in Agent output', $device));
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $nic = $this->json['net_stats'][$device];

        $duplex = [
            1 => 'Half',
            2 => 'Full'
        ];

        $output = sprintf('Network interface [%s] is down!', $device);
        if ($nic['isup']) {
            $output = sprintf(
                'Network interface [%s] is up. Speed: %s, MTU: %s, Duplex mode: %s',
                $device,
                Byte::scaleMbits($nic['speed']),
                $nic['mtu'],
                ($duplex[$nic['duplex']] ?? 'Unknown')
            );
        }

        $this->setOutput($output);
        $this->setStatuscode(NagiosService::STATUS_OK);

        if (!$nic['isup']) {
            if ($downState === 'critical') {
                $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                return;
            } else if ($downState === 'warning') {
                $this->setStatuscode(NagiosService::STATUS_WARNING);
                return;
            } else {
                $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                return;
            }
        }
    }
}
