<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\Converter\Byte;
use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class SwapUsage
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class SwapUsage extends BasePlugin implements PluginInterface {

    public function execute() {
        if ($this->checkError('swap')) {
            return;
        }

        $requiredKeys = [
            'swap' => [
                'total',
                'percent',
                'used',
                'free'
            ]
        ];

        foreach ($requiredKeys as $mainKey => $subKeys) {
            foreach ($subKeys as $subKey) {
                if (!isset($this->json[$mainKey][$subKey])) {
                    $this->setOutput(sprintf('Key "%s.%s" not found in Agent output', $mainKey, $subKey));
                    $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
                    return;
                }
            }
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $warning = $arg;
                    break;
                case 1:
                    $critical = $arg;
                    break;
            }
        }

        $this->addPerfdata(new PerfdataGauge(
            'Swap_Used',
            round($this->json['swap']['percent'], 2),
            '%',
            $warning,
            $critical,
            0,
            100
        ));

        foreach (['used', 'total', 'free'] as $gauge) {
            $this->addPerfdata(new PerfdataGauge(
                $gauge,
                $this->json['swap'][$gauge],
                'B'
            ));
        }

        $this->setOutput(sprintf(
            'Swap usage: %s/%s (%s%%). Used: %s, Free: %s',
            Byte::asHuman($this->json['swap']['used']),
            Byte::asHuman($this->json['swap']['total']),
            $this->json['swap']['percent'],
            Byte::asHuman($this->json['swap']['used']),
            Byte::asHuman($this->json['swap']['free'])
        ));

        if ($this->getRangeStatus($critical, $this->json['swap']['percent'])) {
            $this->setStatuscode(NagiosService::STATUS_CRITICAL);
            return;
        }

        if ($this->getRangeStatus($warning, $this->json['swap']['percent'])) {
            $this->setStatuscode(NagiosService::STATUS_WARNING);
            return;
        }

        $this->setStatuscode(NagiosService::STATUS_OK);
    }
}
