<?php
// Copyright (C) <2018>  <it-novum GmbH>
//
// This file is dual licensed
//
// 1.
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, version 3 of the License.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
// 2.
//  If you purchased an openITCOCKPIT Enterprise Edition you can use this file
//  under the terms of the openITCOCKPIT Enterprise Edition license agreement.
//  License agreement and license key will be shipped with the order
//  confirmation.

namespace itnovum\openITCOCKPIT\Checks\Receiver\Plugins;


use itnovum\openITCOCKPIT\Checks\Receiver\NagiosService;
use itnovum\openITCOCKPIT\Checks\Receiver\ValueObjects\PerfdataGauge;

/**
 * Class WindowsService
 * @package itnovum\openITCOCKPIT\Checks\Receiver\Plugins
 */
class WindowsService extends BasePlugin implements PluginInterface {


    public function execute() {
        if ($this->checkError('windows_services')) {
            return;
        }

        if (!isset($this->json['windows_services']) || !is_array($this->json['windows_services'])) {
            $this->setOutput('Key "windows_services" not found in Agent output');
            return;
        }

        if (!isset($this->pluginConfig['args'])) {
            $this->setOutput('Plugin arguments not found in configuration');
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $match = '';
        $strict = false;
        foreach ($this->pluginConfig['args'] as $key => $arg) {
            switch ($key) {
                case 0:
                    $amountWarning = $arg;
                    break;
                case 1:
                    $amountCritical = $arg;
                    break;
                case 2:
                    $match = trim($arg);
                    break;
                case 3:
                    $strict = (bool)$arg;
                    break;
            }
        }

        $wantedServiceList = [];
        foreach ($this->json['windows_services'] as $service) {
            if ($service['Status'] === 'Running') {
                if ($match === '') {
                    $wantedServiceList[] = $service;
                    continue;
                }

                if ($this->serviceNameMatch($match, $service['Name'], $strict)) {
                    $wantedServiceList[] = $service;
                }
            } else {
                if ($this->serviceNameMatch($match, $service['Name'], $strict)) {
                    $this->setOutput(sprintf(
                        'Service %s is %s',
                        (!empty($service['DisplayName']) ? $service['DisplayName'] : $service['Name']),
                        $service['Status']
                    ));
                    $this->setStatuscode(NagiosService::STATUS_CRITICAL);
                    return;
                }
            }
        }

        if (empty($wantedServiceList)) {
            $this->setOutput(sprintf('No service found matching "%s"!', $match));
            $this->setStatuscode(NagiosService::STATUS_UNKNOWN);
            return;
        }

        $this->addPerfdata(new PerfdataGauge(
            'Amount',
            sizeof($wantedServiceList),
            '',
            $amountWarning,
            $amountCritical,
            0
        ));

        $preformat = '';
        $format = 'Amount: %s services.';

        $output = [];
        foreach ($wantedServiceList as $service) {
            $output[] = sprintf(
                'Service %s is %s',
                (!empty($service['DisplayName']) ? $service['DisplayName'] : $service['Name']),
                $service['Status']
            );
        }

        $this->setStatuscode(NagiosService::STATUS_OK);

        if ($this->getRangeStatus($amountWarning, sizeof($wantedServiceList))) {
            $preformat = 'Service amount warning! ';
            $this->setStatuscode(NagiosService::STATUS_WARNING);
        }

        if ($this->getRangeStatus($amountCritical, sizeof($wantedServiceList))) {
            $preformat = 'Service amount critical! ';
            $this->setStatuscode(NagiosService::STATUS_CRITICAL);
        }

        $outStr = implode(', ', $output);
        $this->setOutput($preformat . $outStr);
    }

    private function serviceNameMatch($match, $processName, $strict) {
        //var_dump($match);
        //echo PHP_EOL;
        if ($strict && trim($match) === trim($processName)) {
            return true;
        }
        if (!$strict && strpos(trim($processName), trim($match)) !== false) {
            return true;
        }
        return false;
    }

}
